# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gnes.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='gnes.proto',
  package='gnes',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\ngnes.proto\x12\x04gnes\x1a\x1fgoogle/protobuf/timestamp.proto\"9\n\x07NdArray\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x11\n\x05shape\x18\x02 \x03(\rB\x02\x10\x01\x12\r\n\x05\x64type\x18\x03 \x01(\t\"\xbc\x01\n\x05\x43hunk\x12\x0e\n\x06\x64oc_id\x18\x01 \x01(\x04\x12\x0e\n\x04text\x18\x02 \x01(\tH\x00\x12\x1d\n\x04\x62lob\x18\x03 \x01(\x0b\x32\r.gnes.NdArrayH\x00\x12\x11\n\toffset_1d\x18\x04 \x01(\r\x12)\n\toffset_nd\x18\x05 \x01(\x0b\x32\x16.gnes.Chunk.Coordinate\x12\x0e\n\x06weight\x18\x06 \x01(\x02\x1a\x1b\n\nCoordinate\x12\r\n\x01x\x18\x01 \x03(\rB\x02\x10\x01\x42\t\n\x07\x63ontent\"\xed\x02\n\x08\x44ocument\x12\x0e\n\x06\x64oc_id\x18\x01 \x01(\x04\x12\x1b\n\x06\x63hunks\x18\x02 \x03(\x0b\x32\x0b.gnes.Chunk\x12\'\n\x10\x63hunk_embeddings\x18\x03 \x01(\x0b\x32\r.gnes.NdArray\x12(\n\x08\x64oc_type\x18\x04 \x01(\x0e\x32\x16.gnes.Document.DocType\x12\x11\n\tmeta_info\x18\x05 \x01(\x0c\x12\x12\n\x08raw_text\x18\x06 \x01(\tH\x00\x12\"\n\traw_image\x18\x07 \x01(\x0b\x32\r.gnes.NdArrayH\x00\x12\"\n\traw_video\x18\x08 \x01(\x0b\x32\r.gnes.NdArrayH\x00\x12\x13\n\traw_bytes\x18\t \x01(\x0cH\x00\x12\x0e\n\x06weight\x18\n \x01(\x02\"A\n\x07\x44ocType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04TEXT\x10\x01\x12\t\n\x05IMAGE\x10\x02\x12\t\n\x05VIDEO\x10\x03\x12\t\n\x05\x41UDIO\x10\x04\x42\n\n\x08raw_data\"\xd4\x01\n\x08\x45nvelope\x12\x11\n\tclient_id\x18\x01 \x01(\t\x12\x12\n\nrequest_id\x18\x02 \x01(\t\x12\x0f\n\x07part_id\x18\x03 \x01(\r\x12\x10\n\x08num_part\x18\x04 \x03(\r\x12\x0f\n\x07timeout\x18\x05 \x01(\r\x12$\n\x06routes\x18\x06 \x03(\x0b\x32\x14.gnes.Envelope.route\x1aG\n\x05route\x12\x0f\n\x07service\x18\x01 \x01(\t\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"y\n\x07Message\x12 \n\x08\x65nvelope\x18\x01 \x01(\x0b\x32\x0e.gnes.Envelope\x12 \n\x07request\x18\x02 \x01(\x0b\x32\r.gnes.RequestH\x00\x12\"\n\x08response\x18\x03 \x01(\x0b\x32\x0e.gnes.ResponseH\x00\x42\x06\n\x04\x62ody\"\xf6\x03\n\x07Request\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12+\n\x05train\x18\x02 \x01(\x0b\x32\x1a.gnes.Request.TrainRequestH\x00\x12+\n\x05index\x18\x03 \x01(\x0b\x32\x1a.gnes.Request.IndexRequestH\x00\x12,\n\x06search\x18\x04 \x01(\x0b\x32\x1a.gnes.Request.QueryRequestH\x00\x12/\n\x07\x63ontrol\x18\x05 \x01(\x0b\x32\x1c.gnes.Request.ControlRequestH\x00\x1a;\n\x0cTrainRequest\x12\x1c\n\x04\x64ocs\x18\x01 \x03(\x0b\x32\x0e.gnes.Document\x12\r\n\x05\x66lush\x18\x02 \x01(\x08\x1a,\n\x0cIndexRequest\x12\x1c\n\x04\x64ocs\x18\x01 \x03(\x0b\x32\x0e.gnes.Document\x1a<\n\x0cQueryRequest\x12\x1d\n\x05query\x18\x01 \x01(\x0b\x32\x0e.gnes.Document\x12\r\n\x05top_k\x18\x02 \x01(\r\x1am\n\x0e\x43ontrolRequest\x12\x35\n\x07\x63ommand\x18\x01 \x01(\x0e\x32$.gnes.Request.ControlRequest.Command\"$\n\x07\x43ommand\x12\r\n\tTERMINATE\x10\x00\x12\n\n\x06STATUS\x10\x01\x42\x06\n\x04\x62ody\"\xc4\x06\n\x08Response\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12-\n\x05train\x18\x02 \x01(\x0b\x32\x1c.gnes.Response.TrainResponseH\x00\x12-\n\x05index\x18\x03 \x01(\x0b\x32\x1c.gnes.Response.IndexResponseH\x00\x12.\n\x06search\x18\x04 \x01(\x0b\x32\x1c.gnes.Response.QueryResponseH\x00\x12\x31\n\x07\x63ontrol\x18\x05 \x01(\x0b\x32\x1e.gnes.Response.ControlResponseH\x00\x1a\x36\n\rTrainResponse\x12%\n\x06status\x18\x01 \x01(\x0e\x32\x15.gnes.Response.Status\x1a\x36\n\rIndexResponse\x12%\n\x06status\x18\x01 \x01(\x0e\x32\x15.gnes.Response.Status\x1a\x38\n\x0f\x43ontrolResponse\x12%\n\x06status\x18\x01 \x01(\x0e\x32\x15.gnes.Response.Status\x1a\x81\x03\n\rQueryResponse\x12%\n\x06status\x18\x01 \x01(\x0e\x32\x15.gnes.Response.Status\x12\r\n\x05top_k\x18\x02 \x01(\r\x12?\n\x0ctopk_results\x18\x03 \x03(\x0b\x32).gnes.Response.QueryResponse.ScoredResult\x12\x39\n\x05level\x18\x04 \x01(\x0e\x32*.gnes.Response.QueryResponse.ResponseLevel\x1a{\n\x0cScoredResult\x12\x1c\n\x05\x63hunk\x18\x01 \x01(\x0b\x32\x0b.gnes.ChunkH\x00\x12\x1d\n\x03\x64oc\x18\x02 \x01(\x0b\x32\x0e.gnes.DocumentH\x00\x12\r\n\x05score\x18\x03 \x01(\x02\x12\x17\n\x0fscore_explained\x18\x04 \x01(\tB\x06\n\x04\x62ody\"A\n\rResponseLevel\x12\t\n\x05\x43HUNK\x10\x00\x12\x17\n\x13\x44OCUMENT_NOT_FILLED\x10\x01\x12\x0c\n\x08\x44OCUMENT\x10\x02\"-\n\x06Status\x12\x0b\n\x07SUCCESS\x10\x00\x12\t\n\x05\x45RROR\x10\x01\x12\x0b\n\x07PENDING\x10\x02\x42\x06\n\x04\x62ody2\xe3\x01\n\x07GnesRPC\x12(\n\x05Train\x12\r.gnes.Request\x1a\x0e.gnes.Response\"\x00\x12(\n\x05Index\x12\r.gnes.Request\x1a\x0e.gnes.Response\"\x00\x12(\n\x05Query\x12\r.gnes.Request\x1a\x0e.gnes.Response\"\x00\x12\'\n\x04\x43\x61ll\x12\r.gnes.Request\x1a\x0e.gnes.Response\"\x00\x12\x31\n\nStreamCall\x12\r.gnes.Request\x1a\x0e.gnes.Response\"\x00(\x01\x30\x01\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])



_DOCUMENT_DOCTYPE = _descriptor.EnumDescriptor(
  name='DocType',
  full_name='gnes.Document.DocType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TEXT', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IMAGE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VIDEO', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AUDIO', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=592,
  serialized_end=657,
)
_sym_db.RegisterEnumDescriptor(_DOCUMENT_DOCTYPE)

_REQUEST_CONTROLREQUEST_COMMAND = _descriptor.EnumDescriptor(
  name='Command',
  full_name='gnes.Request.ControlRequest.Command',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TERMINATE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STATUS', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1468,
  serialized_end=1504,
)
_sym_db.RegisterEnumDescriptor(_REQUEST_CONTROLREQUEST_COMMAND)

_RESPONSE_QUERYRESPONSE_RESPONSELEVEL = _descriptor.EnumDescriptor(
  name='ResponseLevel',
  full_name='gnes.Response.QueryResponse.ResponseLevel',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CHUNK', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DOCUMENT_NOT_FILLED', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DOCUMENT', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2231,
  serialized_end=2296,
)
_sym_db.RegisterEnumDescriptor(_RESPONSE_QUERYRESPONSE_RESPONSELEVEL)

_RESPONSE_STATUS = _descriptor.EnumDescriptor(
  name='Status',
  full_name='gnes.Response.Status',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PENDING', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2298,
  serialized_end=2343,
)
_sym_db.RegisterEnumDescriptor(_RESPONSE_STATUS)


_NDARRAY = _descriptor.Descriptor(
  name='NdArray',
  full_name='gnes.NdArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='gnes.NdArray.data', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='shape', full_name='gnes.NdArray.shape', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dtype', full_name='gnes.NdArray.dtype', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=53,
  serialized_end=110,
)


_CHUNK_COORDINATE = _descriptor.Descriptor(
  name='Coordinate',
  full_name='gnes.Chunk.Coordinate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='gnes.Chunk.Coordinate.x', index=0,
      number=1, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=263,
  serialized_end=290,
)

_CHUNK = _descriptor.Descriptor(
  name='Chunk',
  full_name='gnes.Chunk',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='doc_id', full_name='gnes.Chunk.doc_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='text', full_name='gnes.Chunk.text', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blob', full_name='gnes.Chunk.blob', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset_1d', full_name='gnes.Chunk.offset_1d', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset_nd', full_name='gnes.Chunk.offset_nd', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='weight', full_name='gnes.Chunk.weight', index=5,
      number=6, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CHUNK_COORDINATE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='content', full_name='gnes.Chunk.content',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=113,
  serialized_end=301,
)


_DOCUMENT = _descriptor.Descriptor(
  name='Document',
  full_name='gnes.Document',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='doc_id', full_name='gnes.Document.doc_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='chunks', full_name='gnes.Document.chunks', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='chunk_embeddings', full_name='gnes.Document.chunk_embeddings', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='doc_type', full_name='gnes.Document.doc_type', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='meta_info', full_name='gnes.Document.meta_info', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='raw_text', full_name='gnes.Document.raw_text', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='raw_image', full_name='gnes.Document.raw_image', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='raw_video', full_name='gnes.Document.raw_video', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='raw_bytes', full_name='gnes.Document.raw_bytes', index=8,
      number=9, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='weight', full_name='gnes.Document.weight', index=9,
      number=10, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DOCUMENT_DOCTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='raw_data', full_name='gnes.Document.raw_data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=304,
  serialized_end=669,
)


_ENVELOPE_ROUTE = _descriptor.Descriptor(
  name='route',
  full_name='gnes.Envelope.route',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='service', full_name='gnes.Envelope.route.service', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='gnes.Envelope.route.timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=813,
  serialized_end=884,
)

_ENVELOPE = _descriptor.Descriptor(
  name='Envelope',
  full_name='gnes.Envelope',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='client_id', full_name='gnes.Envelope.client_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='request_id', full_name='gnes.Envelope.request_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='part_id', full_name='gnes.Envelope.part_id', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_part', full_name='gnes.Envelope.num_part', index=3,
      number=4, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timeout', full_name='gnes.Envelope.timeout', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='routes', full_name='gnes.Envelope.routes', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_ENVELOPE_ROUTE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=672,
  serialized_end=884,
)


_MESSAGE = _descriptor.Descriptor(
  name='Message',
  full_name='gnes.Message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='envelope', full_name='gnes.Message.envelope', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='request', full_name='gnes.Message.request', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='response', full_name='gnes.Message.response', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='body', full_name='gnes.Message.body',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=886,
  serialized_end=1007,
)


_REQUEST_TRAINREQUEST = _descriptor.Descriptor(
  name='TrainRequest',
  full_name='gnes.Request.TrainRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='docs', full_name='gnes.Request.TrainRequest.docs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='flush', full_name='gnes.Request.TrainRequest.flush', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1226,
  serialized_end=1285,
)

_REQUEST_INDEXREQUEST = _descriptor.Descriptor(
  name='IndexRequest',
  full_name='gnes.Request.IndexRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='docs', full_name='gnes.Request.IndexRequest.docs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1287,
  serialized_end=1331,
)

_REQUEST_QUERYREQUEST = _descriptor.Descriptor(
  name='QueryRequest',
  full_name='gnes.Request.QueryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='query', full_name='gnes.Request.QueryRequest.query', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='top_k', full_name='gnes.Request.QueryRequest.top_k', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1333,
  serialized_end=1393,
)

_REQUEST_CONTROLREQUEST = _descriptor.Descriptor(
  name='ControlRequest',
  full_name='gnes.Request.ControlRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='command', full_name='gnes.Request.ControlRequest.command', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _REQUEST_CONTROLREQUEST_COMMAND,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1395,
  serialized_end=1504,
)

_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='gnes.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='request_id', full_name='gnes.Request.request_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='train', full_name='gnes.Request.train', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='index', full_name='gnes.Request.index', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='search', full_name='gnes.Request.search', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='control', full_name='gnes.Request.control', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_REQUEST_TRAINREQUEST, _REQUEST_INDEXREQUEST, _REQUEST_QUERYREQUEST, _REQUEST_CONTROLREQUEST, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='body', full_name='gnes.Request.body',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1010,
  serialized_end=1512,
)


_RESPONSE_TRAINRESPONSE = _descriptor.Descriptor(
  name='TrainResponse',
  full_name='gnes.Response.TrainResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='gnes.Response.TrainResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1740,
  serialized_end=1794,
)

_RESPONSE_INDEXRESPONSE = _descriptor.Descriptor(
  name='IndexResponse',
  full_name='gnes.Response.IndexResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='gnes.Response.IndexResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1796,
  serialized_end=1850,
)

_RESPONSE_CONTROLRESPONSE = _descriptor.Descriptor(
  name='ControlResponse',
  full_name='gnes.Response.ControlResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='gnes.Response.ControlResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1852,
  serialized_end=1908,
)

_RESPONSE_QUERYRESPONSE_SCOREDRESULT = _descriptor.Descriptor(
  name='ScoredResult',
  full_name='gnes.Response.QueryResponse.ScoredResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chunk', full_name='gnes.Response.QueryResponse.ScoredResult.chunk', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='doc', full_name='gnes.Response.QueryResponse.ScoredResult.doc', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='gnes.Response.QueryResponse.ScoredResult.score', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score_explained', full_name='gnes.Response.QueryResponse.ScoredResult.score_explained', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='body', full_name='gnes.Response.QueryResponse.ScoredResult.body',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=2106,
  serialized_end=2229,
)

_RESPONSE_QUERYRESPONSE = _descriptor.Descriptor(
  name='QueryResponse',
  full_name='gnes.Response.QueryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='gnes.Response.QueryResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='top_k', full_name='gnes.Response.QueryResponse.top_k', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='topk_results', full_name='gnes.Response.QueryResponse.topk_results', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='level', full_name='gnes.Response.QueryResponse.level', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_RESPONSE_QUERYRESPONSE_SCOREDRESULT, ],
  enum_types=[
    _RESPONSE_QUERYRESPONSE_RESPONSELEVEL,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1911,
  serialized_end=2296,
)

_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='gnes.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='request_id', full_name='gnes.Response.request_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='train', full_name='gnes.Response.train', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='index', full_name='gnes.Response.index', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='search', full_name='gnes.Response.search', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='control', full_name='gnes.Response.control', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_RESPONSE_TRAINRESPONSE, _RESPONSE_INDEXRESPONSE, _RESPONSE_CONTROLRESPONSE, _RESPONSE_QUERYRESPONSE, ],
  enum_types=[
    _RESPONSE_STATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='body', full_name='gnes.Response.body',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1515,
  serialized_end=2351,
)

_CHUNK_COORDINATE.containing_type = _CHUNK
_CHUNK.fields_by_name['blob'].message_type = _NDARRAY
_CHUNK.fields_by_name['offset_nd'].message_type = _CHUNK_COORDINATE
_CHUNK.oneofs_by_name['content'].fields.append(
  _CHUNK.fields_by_name['text'])
_CHUNK.fields_by_name['text'].containing_oneof = _CHUNK.oneofs_by_name['content']
_CHUNK.oneofs_by_name['content'].fields.append(
  _CHUNK.fields_by_name['blob'])
_CHUNK.fields_by_name['blob'].containing_oneof = _CHUNK.oneofs_by_name['content']
_DOCUMENT.fields_by_name['chunks'].message_type = _CHUNK
_DOCUMENT.fields_by_name['chunk_embeddings'].message_type = _NDARRAY
_DOCUMENT.fields_by_name['doc_type'].enum_type = _DOCUMENT_DOCTYPE
_DOCUMENT.fields_by_name['raw_image'].message_type = _NDARRAY
_DOCUMENT.fields_by_name['raw_video'].message_type = _NDARRAY
_DOCUMENT_DOCTYPE.containing_type = _DOCUMENT
_DOCUMENT.oneofs_by_name['raw_data'].fields.append(
  _DOCUMENT.fields_by_name['raw_text'])
_DOCUMENT.fields_by_name['raw_text'].containing_oneof = _DOCUMENT.oneofs_by_name['raw_data']
_DOCUMENT.oneofs_by_name['raw_data'].fields.append(
  _DOCUMENT.fields_by_name['raw_image'])
_DOCUMENT.fields_by_name['raw_image'].containing_oneof = _DOCUMENT.oneofs_by_name['raw_data']
_DOCUMENT.oneofs_by_name['raw_data'].fields.append(
  _DOCUMENT.fields_by_name['raw_video'])
_DOCUMENT.fields_by_name['raw_video'].containing_oneof = _DOCUMENT.oneofs_by_name['raw_data']
_DOCUMENT.oneofs_by_name['raw_data'].fields.append(
  _DOCUMENT.fields_by_name['raw_bytes'])
_DOCUMENT.fields_by_name['raw_bytes'].containing_oneof = _DOCUMENT.oneofs_by_name['raw_data']
_ENVELOPE_ROUTE.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ENVELOPE_ROUTE.containing_type = _ENVELOPE
_ENVELOPE.fields_by_name['routes'].message_type = _ENVELOPE_ROUTE
_MESSAGE.fields_by_name['envelope'].message_type = _ENVELOPE
_MESSAGE.fields_by_name['request'].message_type = _REQUEST
_MESSAGE.fields_by_name['response'].message_type = _RESPONSE
_MESSAGE.oneofs_by_name['body'].fields.append(
  _MESSAGE.fields_by_name['request'])
_MESSAGE.fields_by_name['request'].containing_oneof = _MESSAGE.oneofs_by_name['body']
_MESSAGE.oneofs_by_name['body'].fields.append(
  _MESSAGE.fields_by_name['response'])
_MESSAGE.fields_by_name['response'].containing_oneof = _MESSAGE.oneofs_by_name['body']
_REQUEST_TRAINREQUEST.fields_by_name['docs'].message_type = _DOCUMENT
_REQUEST_TRAINREQUEST.containing_type = _REQUEST
_REQUEST_INDEXREQUEST.fields_by_name['docs'].message_type = _DOCUMENT
_REQUEST_INDEXREQUEST.containing_type = _REQUEST
_REQUEST_QUERYREQUEST.fields_by_name['query'].message_type = _DOCUMENT
_REQUEST_QUERYREQUEST.containing_type = _REQUEST
_REQUEST_CONTROLREQUEST.fields_by_name['command'].enum_type = _REQUEST_CONTROLREQUEST_COMMAND
_REQUEST_CONTROLREQUEST.containing_type = _REQUEST
_REQUEST_CONTROLREQUEST_COMMAND.containing_type = _REQUEST_CONTROLREQUEST
_REQUEST.fields_by_name['train'].message_type = _REQUEST_TRAINREQUEST
_REQUEST.fields_by_name['index'].message_type = _REQUEST_INDEXREQUEST
_REQUEST.fields_by_name['search'].message_type = _REQUEST_QUERYREQUEST
_REQUEST.fields_by_name['control'].message_type = _REQUEST_CONTROLREQUEST
_REQUEST.oneofs_by_name['body'].fields.append(
  _REQUEST.fields_by_name['train'])
_REQUEST.fields_by_name['train'].containing_oneof = _REQUEST.oneofs_by_name['body']
_REQUEST.oneofs_by_name['body'].fields.append(
  _REQUEST.fields_by_name['index'])
_REQUEST.fields_by_name['index'].containing_oneof = _REQUEST.oneofs_by_name['body']
_REQUEST.oneofs_by_name['body'].fields.append(
  _REQUEST.fields_by_name['search'])
_REQUEST.fields_by_name['search'].containing_oneof = _REQUEST.oneofs_by_name['body']
_REQUEST.oneofs_by_name['body'].fields.append(
  _REQUEST.fields_by_name['control'])
_REQUEST.fields_by_name['control'].containing_oneof = _REQUEST.oneofs_by_name['body']
_RESPONSE_TRAINRESPONSE.fields_by_name['status'].enum_type = _RESPONSE_STATUS
_RESPONSE_TRAINRESPONSE.containing_type = _RESPONSE
_RESPONSE_INDEXRESPONSE.fields_by_name['status'].enum_type = _RESPONSE_STATUS
_RESPONSE_INDEXRESPONSE.containing_type = _RESPONSE
_RESPONSE_CONTROLRESPONSE.fields_by_name['status'].enum_type = _RESPONSE_STATUS
_RESPONSE_CONTROLRESPONSE.containing_type = _RESPONSE
_RESPONSE_QUERYRESPONSE_SCOREDRESULT.fields_by_name['chunk'].message_type = _CHUNK
_RESPONSE_QUERYRESPONSE_SCOREDRESULT.fields_by_name['doc'].message_type = _DOCUMENT
_RESPONSE_QUERYRESPONSE_SCOREDRESULT.containing_type = _RESPONSE_QUERYRESPONSE
_RESPONSE_QUERYRESPONSE_SCOREDRESULT.oneofs_by_name['body'].fields.append(
  _RESPONSE_QUERYRESPONSE_SCOREDRESULT.fields_by_name['chunk'])
_RESPONSE_QUERYRESPONSE_SCOREDRESULT.fields_by_name['chunk'].containing_oneof = _RESPONSE_QUERYRESPONSE_SCOREDRESULT.oneofs_by_name['body']
_RESPONSE_QUERYRESPONSE_SCOREDRESULT.oneofs_by_name['body'].fields.append(
  _RESPONSE_QUERYRESPONSE_SCOREDRESULT.fields_by_name['doc'])
_RESPONSE_QUERYRESPONSE_SCOREDRESULT.fields_by_name['doc'].containing_oneof = _RESPONSE_QUERYRESPONSE_SCOREDRESULT.oneofs_by_name['body']
_RESPONSE_QUERYRESPONSE.fields_by_name['status'].enum_type = _RESPONSE_STATUS
_RESPONSE_QUERYRESPONSE.fields_by_name['topk_results'].message_type = _RESPONSE_QUERYRESPONSE_SCOREDRESULT
_RESPONSE_QUERYRESPONSE.fields_by_name['level'].enum_type = _RESPONSE_QUERYRESPONSE_RESPONSELEVEL
_RESPONSE_QUERYRESPONSE.containing_type = _RESPONSE
_RESPONSE_QUERYRESPONSE_RESPONSELEVEL.containing_type = _RESPONSE_QUERYRESPONSE
_RESPONSE.fields_by_name['train'].message_type = _RESPONSE_TRAINRESPONSE
_RESPONSE.fields_by_name['index'].message_type = _RESPONSE_INDEXRESPONSE
_RESPONSE.fields_by_name['search'].message_type = _RESPONSE_QUERYRESPONSE
_RESPONSE.fields_by_name['control'].message_type = _RESPONSE_CONTROLRESPONSE
_RESPONSE_STATUS.containing_type = _RESPONSE
_RESPONSE.oneofs_by_name['body'].fields.append(
  _RESPONSE.fields_by_name['train'])
_RESPONSE.fields_by_name['train'].containing_oneof = _RESPONSE.oneofs_by_name['body']
_RESPONSE.oneofs_by_name['body'].fields.append(
  _RESPONSE.fields_by_name['index'])
_RESPONSE.fields_by_name['index'].containing_oneof = _RESPONSE.oneofs_by_name['body']
_RESPONSE.oneofs_by_name['body'].fields.append(
  _RESPONSE.fields_by_name['search'])
_RESPONSE.fields_by_name['search'].containing_oneof = _RESPONSE.oneofs_by_name['body']
_RESPONSE.oneofs_by_name['body'].fields.append(
  _RESPONSE.fields_by_name['control'])
_RESPONSE.fields_by_name['control'].containing_oneof = _RESPONSE.oneofs_by_name['body']
DESCRIPTOR.message_types_by_name['NdArray'] = _NDARRAY
DESCRIPTOR.message_types_by_name['Chunk'] = _CHUNK
DESCRIPTOR.message_types_by_name['Document'] = _DOCUMENT
DESCRIPTOR.message_types_by_name['Envelope'] = _ENVELOPE
DESCRIPTOR.message_types_by_name['Message'] = _MESSAGE
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

NdArray = _reflection.GeneratedProtocolMessageType('NdArray', (_message.Message,), {
  'DESCRIPTOR' : _NDARRAY,
  '__module__' : 'gnes_pb2'
  # @@protoc_insertion_point(class_scope:gnes.NdArray)
  })
_sym_db.RegisterMessage(NdArray)

Chunk = _reflection.GeneratedProtocolMessageType('Chunk', (_message.Message,), {

  'Coordinate' : _reflection.GeneratedProtocolMessageType('Coordinate', (_message.Message,), {
    'DESCRIPTOR' : _CHUNK_COORDINATE,
    '__module__' : 'gnes_pb2'
    # @@protoc_insertion_point(class_scope:gnes.Chunk.Coordinate)
    })
  ,
  'DESCRIPTOR' : _CHUNK,
  '__module__' : 'gnes_pb2'
  # @@protoc_insertion_point(class_scope:gnes.Chunk)
  })
_sym_db.RegisterMessage(Chunk)
_sym_db.RegisterMessage(Chunk.Coordinate)

Document = _reflection.GeneratedProtocolMessageType('Document', (_message.Message,), {
  'DESCRIPTOR' : _DOCUMENT,
  '__module__' : 'gnes_pb2'
  # @@protoc_insertion_point(class_scope:gnes.Document)
  })
_sym_db.RegisterMessage(Document)

Envelope = _reflection.GeneratedProtocolMessageType('Envelope', (_message.Message,), {

  'route' : _reflection.GeneratedProtocolMessageType('route', (_message.Message,), {
    'DESCRIPTOR' : _ENVELOPE_ROUTE,
    '__module__' : 'gnes_pb2'
    # @@protoc_insertion_point(class_scope:gnes.Envelope.route)
    })
  ,
  'DESCRIPTOR' : _ENVELOPE,
  '__module__' : 'gnes_pb2'
  # @@protoc_insertion_point(class_scope:gnes.Envelope)
  })
_sym_db.RegisterMessage(Envelope)
_sym_db.RegisterMessage(Envelope.route)

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), {
  'DESCRIPTOR' : _MESSAGE,
  '__module__' : 'gnes_pb2'
  # @@protoc_insertion_point(class_scope:gnes.Message)
  })
_sym_db.RegisterMessage(Message)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), {

  'TrainRequest' : _reflection.GeneratedProtocolMessageType('TrainRequest', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_TRAINREQUEST,
    '__module__' : 'gnes_pb2'
    # @@protoc_insertion_point(class_scope:gnes.Request.TrainRequest)
    })
  ,

  'IndexRequest' : _reflection.GeneratedProtocolMessageType('IndexRequest', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_INDEXREQUEST,
    '__module__' : 'gnes_pb2'
    # @@protoc_insertion_point(class_scope:gnes.Request.IndexRequest)
    })
  ,

  'QueryRequest' : _reflection.GeneratedProtocolMessageType('QueryRequest', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_QUERYREQUEST,
    '__module__' : 'gnes_pb2'
    # @@protoc_insertion_point(class_scope:gnes.Request.QueryRequest)
    })
  ,

  'ControlRequest' : _reflection.GeneratedProtocolMessageType('ControlRequest', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_CONTROLREQUEST,
    '__module__' : 'gnes_pb2'
    # @@protoc_insertion_point(class_scope:gnes.Request.ControlRequest)
    })
  ,
  'DESCRIPTOR' : _REQUEST,
  '__module__' : 'gnes_pb2'
  # @@protoc_insertion_point(class_scope:gnes.Request)
  })
_sym_db.RegisterMessage(Request)
_sym_db.RegisterMessage(Request.TrainRequest)
_sym_db.RegisterMessage(Request.IndexRequest)
_sym_db.RegisterMessage(Request.QueryRequest)
_sym_db.RegisterMessage(Request.ControlRequest)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {

  'TrainResponse' : _reflection.GeneratedProtocolMessageType('TrainResponse', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_TRAINRESPONSE,
    '__module__' : 'gnes_pb2'
    # @@protoc_insertion_point(class_scope:gnes.Response.TrainResponse)
    })
  ,

  'IndexResponse' : _reflection.GeneratedProtocolMessageType('IndexResponse', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_INDEXRESPONSE,
    '__module__' : 'gnes_pb2'
    # @@protoc_insertion_point(class_scope:gnes.Response.IndexResponse)
    })
  ,

  'ControlResponse' : _reflection.GeneratedProtocolMessageType('ControlResponse', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_CONTROLRESPONSE,
    '__module__' : 'gnes_pb2'
    # @@protoc_insertion_point(class_scope:gnes.Response.ControlResponse)
    })
  ,

  'QueryResponse' : _reflection.GeneratedProtocolMessageType('QueryResponse', (_message.Message,), {

    'ScoredResult' : _reflection.GeneratedProtocolMessageType('ScoredResult', (_message.Message,), {
      'DESCRIPTOR' : _RESPONSE_QUERYRESPONSE_SCOREDRESULT,
      '__module__' : 'gnes_pb2'
      # @@protoc_insertion_point(class_scope:gnes.Response.QueryResponse.ScoredResult)
      })
    ,
    'DESCRIPTOR' : _RESPONSE_QUERYRESPONSE,
    '__module__' : 'gnes_pb2'
    # @@protoc_insertion_point(class_scope:gnes.Response.QueryResponse)
    })
  ,
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'gnes_pb2'
  # @@protoc_insertion_point(class_scope:gnes.Response)
  })
_sym_db.RegisterMessage(Response)
_sym_db.RegisterMessage(Response.TrainResponse)
_sym_db.RegisterMessage(Response.IndexResponse)
_sym_db.RegisterMessage(Response.ControlResponse)
_sym_db.RegisterMessage(Response.QueryResponse)
_sym_db.RegisterMessage(Response.QueryResponse.ScoredResult)


_NDARRAY.fields_by_name['shape']._options = None
_CHUNK_COORDINATE.fields_by_name['x']._options = None

_GNESRPC = _descriptor.ServiceDescriptor(
  name='GnesRPC',
  full_name='gnes.GnesRPC',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=2354,
  serialized_end=2581,
  methods=[
  _descriptor.MethodDescriptor(
    name='Train',
    full_name='gnes.GnesRPC.Train',
    index=0,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Index',
    full_name='gnes.GnesRPC.Index',
    index=1,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Query',
    full_name='gnes.GnesRPC.Query',
    index=2,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Call',
    full_name='gnes.GnesRPC.Call',
    index=3,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='StreamCall',
    full_name='gnes.GnesRPC.StreamCall',
    index=4,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_GNESRPC)

DESCRIPTOR.services_by_name['GnesRPC'] = _GNESRPC

# @@protoc_insertion_point(module_scope)
